﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Xml.Linq;
using Microsoft.VisualBasic;

namespace Azyobuzi.りすにんぐついったー
{
    //http://twitter.com/pixy_tom/status/27318124014
    public class NpmHashtagCreater
    {
        public NpmHashtagCreater(string yahooappid)
        {
            this.YahooAppId = yahooappid;
        }

        public string YahooAppId { set; get; }

        public const string BITLY_USER_NAME = "npmhashtag";
        public const string BITLY_API_KEY = "R_89a9ed5863119608a2b42b23a9eec053";

        private string CreateUriForFuriganaApi(string sentence)
        {
            return string.Format("http://jlp.yahooapis.jp/FuriganaService/V1/furigana?appid={0}&sentence={1}",
                YahooAppId, Uri.EscapeDataString(sentence));
        }

        private string CreateUriForBitlyShortenApi(string longUrl)
        {
            return string.Format("http://api.bit.ly/v3/shorten?login={0}&apikey={1}&longUrl={2}&format=xml",
                BITLY_USER_NAME, BITLY_API_KEY, longUrl);
        }

        public string CreateHashtag(string artist, string album)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(artist) || string.IsNullOrWhiteSpace(album)) return null;
                using (WebClient wc = new WebClient())
                {
                    wc.Encoding = Encoding.UTF8;

                    string tagString = artist + album;
                    string temp = System.Text.RegularExpressions.Regex.Replace(
                        Strings.StrConv(tagString, VbStrConv.Lowercase | VbStrConv.Narrow),
                        @"[,\.\/\\;:\]\[@`\{\}\*\+_\?><!""#\$%&'\(\)\-=\^~\|．。、「」\s]", "");

                    XElement xml;
                    string uri = CreateUriForFuriganaApi(temp);
                    xml = XElement.Parse(wc.DownloadString(uri));
                    var ns = xml.Name.Namespace;
                    xml.Descendants(ns + "Word").Where(o => o.Element(ns + "Furigana") != null).ToList()
                        .ForEach(xe => temp = temp.Replace(xe.Element(ns + "Surface").Value, xe.Element(ns + "Furigana").Value));

                    temp = temp.Replace('が', 'か').Replace('ぎ', 'き').Replace('ぐ', 'く').Replace('げ', 'け').Replace('ご', 'こ')
                        .Replace('ざ', 'さ').Replace('じ', 'し').Replace('ず', 'す').Replace('ぜ', 'せ').Replace('ぞ', 'そ')
                        .Replace('だ', 'た').Replace('ぢ', 'ち').Replace('づ', 'つ').Replace('で', 'て').Replace('ど', 'と')
                        .Replace('ば', 'は').Replace('び', 'ひ').Replace('ぶ', 'ふ').Replace('べ', 'へ').Replace('ぼ', 'ほ')
                        .Replace('ぱ', 'は').Replace('ぴ', 'ひ').Replace('ぷ', 'ふ').Replace('ぺ', 'へ').Replace('ぽ', 'ほ')
                        .Replace('ぁ', 'あ').Replace('ぃ', 'い').Replace('ぅ', 'う').Replace('ぇ', 'え').Replace('ぉ', 'お')
                        .Replace('ゃ', 'や').Replace('ゅ', 'ゆ').Replace('ょ', 'よ').Replace("ヴ", "ふ");

                    uri = CreateUriForFuriganaApi(temp);
                    xml = XElement.Parse(wc.DownloadString(uri));
                    ns = xml.Name.Namespace;
                    xml.Descendants(ns + "Word").Where(o => o.Element(ns + "Roman") != null).ToList()
                        .ForEach(xe => temp = temp.Replace(xe.Element(ns + "Surface").Value, xe.Element(ns + "Roman").Value));

                    string tempUri = "http://www.amazon.com/" + Uri.EscapeDataString(temp);
                    uri = CreateUriForBitlyShortenApi(tempUri);
                    xml = XElement.Parse(wc.DownloadString(uri));

                    return "#NPM" + xml.Descendants("url").First().Value.Substring("http://amzn.to/".Length);
                }
            }
            catch
            {
                return null;
            }
        }
    }

    [Serializable]
    public class NpmHashtagCache : ICollection<CacheItem>
    {
        public IEnumerator<CacheItem> GetEnumerator()
        {
            return innerList.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        private List<CacheItem> innerList = new List<CacheItem>();

        private int capacity = 50;
        public int Capacity
        {
            set
            {
                capacity = value;
                AutoRemove();
            }
            get
            {
                return capacity;
            }
        }

        private void AutoRemove()
        {
            var collection = innerList.OrderBy(o => o.LastAccess).ToList();
            int diff = collection.Count - Capacity;
            for (int i = 1; i <= diff; i++)
            {
                collection.RemoveAt(0);
            }
            innerList = collection;
        }

        void ICollection<CacheItem>.Add(CacheItem item)
        {
            innerList.Add(item);
            AutoRemove();
        }

        //シリアライザー対策
        public void Add(object item)
        {
            innerList.Add((CacheItem)item);
            AutoRemove();
        }

        public void Clear()
        {
            innerList.Clear();
        }

        bool ICollection<CacheItem>.Contains(CacheItem item)
        {
            return innerList.Contains(item);
        }

        void ICollection<CacheItem>.CopyTo(CacheItem[] array, int arrayIndex)
        {
            innerList.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return innerList.Count; }
        }

        bool ICollection<CacheItem>.IsReadOnly
        {
            get { return ((ICollection<CacheItem>)innerList).IsReadOnly; }
        }

        bool ICollection<CacheItem>.Remove(CacheItem item)
        {
            return innerList.Remove(item);
        }

        public void Add(string artist, string album, string hashtag)
        {
            if (string.IsNullOrWhiteSpace(artist) || string.IsNullOrWhiteSpace(album) || string.IsNullOrWhiteSpace(hashtag))
                throw new ArgumentException("引数はすべて指定してください。");
            if (!hashtag.StartsWith("#NPM")) throw new ArgumentException(@"hashtagは""#NPM""から始まらなければいけません。");
            int index = innerList.IndexOf(this[artist, album]);
            if (index == -1)
                innerList.Add(new CacheItem() { Artist = artist, Album = album, Hashtag = hashtag, LastAccess = DateTime.Now });
            else innerList[index].LastAccess = DateTime.Now;
            AutoRemove();
        }

        public bool Contains(string artist, string album)
        {
            return this[artist, album] != null;
        }

        private CacheItem this[string artist, string album]
        {
            get
            {
                return innerList.FirstOrDefault(o =>
                    artist.Equals(o.Artist, StringComparison.InvariantCultureIgnoreCase) &&
                    album.Equals(o.Album, StringComparison.InvariantCultureIgnoreCase));
            }
        }

        public CacheItem GetItem(string artist, string album)
        {
            var item = this[artist, album];
            if (item == null) return null;
            else
            {
                item.LastAccess = DateTime.Now;
                return item;
            }
        }
    }

    [Serializable]
    public class CacheItem
    {
        public string Artist { set; get; }
        public string Album { set; get; }
        public string Hashtag { set; get; }
        public DateTime LastAccess { set; get; }
    }
}
